import os
import sys
import subprocess
from pathlib import Path

def update():
    """
    Updates Nexy to the latest version using pip
    """
    try:
        subprocess.check_call([sys.executable, "-m", "pip", "install", "--upgrade", "nexy"])
        print("✨ Nexy has been updated successfully!")
    except subprocess.CalledProcessError as e:
        print("❌ Error updating Nexy:", e)
        sys.exit(1)



from fastapi.responses import  HTMLResponse
from jinja2 import Environment, FileSystemLoader, TemplateNotFound

env = Environment(loader=FileSystemLoader("."), enable_async=True)

def useView(data,path):
    
    
    print("data is:", data)
    print("path is :", path)
    try:
        
        template = env.get_template(f"{path}/view.html")
        return template.render(data)
    
    except TemplateNotFound:
        return HTMLResponse(content=f"Template non trouvé : {path}/view.html", status_code=404)
    except Exception as e:
        return HTMLResponse(content=f"Erreur lors du rendu du template : {str(e)}", status_code=500)




import os
import sys
from pathlib import Path
from fastapi import APIRouter, FastAPI
from fastapi.staticfiles import StaticFiles
from scalar_fastapi import get_scalar_api_reference
from .router import Router

# SVG icon data
SVG_DATA_URI = """data:image/svg+xml;base64,<svg width='100' height='100' viewBox='0 0 100 100' fill='none' xmlns='http://www.w3.org/2000/svg'>
        <rect width='100' height='100' fill='#CBECE3'/>
        <path d='M27 78V22H30.1379L69.2414 60.0575V22H72.2184V78H27Z' fill='#1CB68D'/>
        </svg>
        """


def Nexy(title: str = None, favicon: str = SVG_DATA_URI, **args) -> FastAPI:
    """
    Creates a FastAPI instance with base configurations.

    :param title: Application title (defaults to current directory name).
    :param favicon: Icon URL or data (defaults to SVG data defined above).
    :param args: Additional arguments to pass to FastAPI.
    :return: Configured FastAPI instance.
    """
    
    # If no title is passed, use current directory name
    if title is None:
        title = Path.cwd().name 

    # Create FastAPI instance
    app: FastAPI = FastAPI(
        title=title,
        docs_url=None,  # Disable standard docs URL
        redoc_url=None,  # Disable standard redoc URL
        **args
    )
    

    
    @app.get("/docs", include_in_schema=False)
    async def scalar_html():  
        """
        Provides a custom OpenAPI documentation view with defined icon.
        """
        return get_scalar_api_reference(
            openapi_url=app.openapi_url,
            title=app.title,
            scalar_favicon_url=favicon
        )

    # Mount static folder if it exists
    static_dir = "public"
    if os.path.exists(static_dir):
        app.mount("/public", StaticFiles(directory=static_dir), name="public")

    # Include router
    Router(app)

    # Configure cache directory
    cache_dir = Path('./__pycache__/nexy')
    cache_dir.mkdir(parents=True, exist_ok=True)
    sys.pycache_prefix = str(cache_dir)

    return app




from functools import wraps
from typing import Any, Callable, Dict, Optional, Sequence, Type, TypeVar, Union

from fastapi.datastructures import Default, DefaultPlaceholder
from fastapi import Depends 
from fastapi import Response as FastAPIResponse
from fastapi.responses import JSONResponse 
from fastapi.datastructures import Default
from fastapi.types import IncEx





T = TypeVar("T")
DependencyType = Union[Callable[..., Any], Type[Any]]

def Injectable() -> Any:
    def decorator(func):
        @wraps(func)
        def wrapper(*args, **kwargs):
            # Appeler la fonction originale et obtenir son résultat
            result = func(*args, **kwargs)
            return Depends(result)
        return wrapper
    return decorator



def Inject(dependencies:Sequence[Depends] | None = None):
    def decorator(func):
        func.params = {
            "dependencies": dependencies,
        }
        @wraps(func)
        def wrapper(*args, **kwargs):
            # Appeler la fonction originale et obtenir son résultat
            result = func(*args, **kwargs)
            return result

        return wrapper
    return decorator



def CustomResponse(
        model:Any = Default(None),
        response_map: Dict[int | str, Dict[str, Any]] | None = None,       
        model_include: IncEx | None = None,
        model_exclude: IncEx | None = None,
        model_by_alias: bool = True,
        model_exclude_unset: bool = False,
        model_exclude_defaults: bool = False,
        model_exclude_none: bool = False,
        type: type[FastAPIResponse] | DefaultPlaceholder = Default(JSONResponse), # type: ignore
         ):
    def decorator(func):
        func.params = {
            "response_model": model,
            "response_model_include": model_include,
            "response_model_exclude": model_exclude,
            "response_model_by_alias": model_by_alias,
            "response_model_exclude_unset": model_exclude_unset,
            "response_model_exclude_defaults": model_exclude_defaults,
            "response_model_exclude_none": model_exclude_none,
            "response_class": type,
            "responses": response_map
        }
        @wraps(func)
        def wrapper(*args, **kwargs):
            # Appeler la fonction originale et obtenir son résultat
            result = func(*args, **kwargs)
            return result

        return wrapper
    return decorator


def Describe(
      summary: str | None = None,
      decription: str | None = None,
      response: str = "Successful Response",
      ):
    def decorator(func):
        func.params = {
            "summary": summary,
            "description": decription,
            "response_description": response,

        }
        @wraps(func)
        def wrapper(*args, **kwargs):
            # Appeler la fonction originale et obtenir son résultat
            result = func(*args, **kwargs)
            return result

        return wrapper
    return decorator

def Config(
        status_code: int | None = None,
        deprecated: bool | None = None,
        operation_id: str | None = None,
        include_in_schema: bool = True,
        name: str | None = None,
        openapi_extra: Dict[str, Any] | None = None,
        ):
    def decorator(func):
        func.params = {
            "deprecated": deprecated,
            "operation_id": operation_id,
            "name": name,
            "include_in_schema": include_in_schema,
            "openapi_extra": openapi_extra,
            "status_code": status_code,
        }
        @wraps(func)
        def wrapper(*args, **kwargs):
            # Appeler la fonction originale et obtenir son résultat
            result = func(*args, **kwargs)
            return result

        return wrapper
    return decorator


def Controller():
    pass

def Get():
    pass

def Post():
    pass

def Put():
    pass

def Delete():
    pass




from typing import List, Optional
from nexy.cli.core.models import ProjectType, Database, ORM, TestFramework

class ProjectBuilder:
    def __init__(self, project_name: str):
        self.project_name = project_name
        self.project_type: Optional[ProjectType] = None
        self.database: Optional[Database] = None
        self.orm: Optional[ORM] = None
        self.test_framework: Optional[TestFramework] = None
        self.features: List[str] = []

    def set_project_type(self, project_type: ProjectType) -> 'ProjectBuilder':
        self.project_type = project_type
        return self

    def set_database(self, database: Database) -> 'ProjectBuilder':
        self.database = database
        return self

    def set_orm(self, orm: ORM) -> 'ProjectBuilder':
        self.orm = orm
        return self

    def set_test_framework(self, test_framework: TestFramework) -> 'ProjectBuilder':
        self.test_framework = test_framework
        return self

    def add_feature(self, feature: str) -> 'ProjectBuilder':
        self.features.append(feature)
        return self
    

    def build(self) -> None:
        """Crée la structure du projet avec les configurations spécifiées"""
        from nexy.cli.core.utils import create_project_structure, setup_virtualenv
        
        create_project_structure(
            project_name=self.project_name,
            project_type=self.project_type or ProjectType.API,
            database=self.database or Database.NONE,
            orm=self.orm or ORM.NONE,
            test_framework=self.test_framework or TestFramework.NONE,
            features=self.features
        ) 
        setup_virtualenv(self.project_name,'nexy_env', 'requirements.txt')